# File tres/Scripts/btres.cl
# June 9, 2009
# By Doug Mink, SAO Telescope Data Center

# BTRES - Process multiple similar TRES dark or bias images from raw data files

procedure btres (images)

string	images=""	{prompt="Images to process"}
bool	fixpreamp=no	{prompt="Run tpreamp to fix time constant (yes or no)"}
bool	fixpix=yes	{prompt="Fix bad pixels (yes or no)"}
bool	fixbias=no	{prompt="Fix bias section (yes or no)"}
real	gaincorr=0.95	{prompt="Gain correction for amp2 to amp1"}
bool	flatten=no	{prompt="Run TFLAT to Flatten image (yes or no)"}
string	normfile=""	{prompt="Pixel to pixel tmakeflat normalization file"}
string	normdir=""	{prompt="Directory for normalization file"}
bool	cosmic=yes	{prompt="Remove cosmic rays (yes or no)"}
real	threshold=10.0	{prompt="Cosmic ray threshold in sigma"}
char	sumspec="median"	{prompt="Process combined 2-D file (sum mean median only(sum) no)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	delimage=no	{prompt="Delete processed image file (yes or no)"}
bool	saveraw=yes	{prompt="Save unprocessed image files (y or n)"}
bool	saveold=no	{prompt="Save intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	imroot, imin, impath
file	imout, ims, cdir, tempin, tempout
bool	vb, dt, sr, sv, disp, rc, delim, avg
real	ct
string	sum

	vb = verbose
	dt = deltemp
	sv = saveold
	sr = saveraw
	ims = images
	delim = delimage
	ct = threshold
	sum = sumspec

	if (substr (ims,1,1) == "@")
	    fileroot (substr (ims,2,100))
	else
	    fileroot (ims)
	trssum.outroot = fileroot.rootname

	if (ims == "") {
	    print ("BTRES: Reduce multiple similar TRES dark or bias images, removing cosmic rays")
	    print ("Usage: btres images with both amps to be corrected")
	    return
	    }

# If input image is a list prepend @ if not already there
	smatch (ims, ".list")
	if (smatch.found) {
	    if (substr (ims,1,1) != '@')
		ims = "@" // images
	    }

	tcheck
	if (tcheck.rawdir)
	    trsdate (tcheck.datedir)

#  Make list file from input file parameter
	trsfiles.instdir = ctres.instdir
	trsfiles.datedir = ctres.datedir
	trsfiles.copyfile = yes
	trsfiles.overwrite = no
	trsfiles (ims)
	if (trsfiles.nfiles == 0) {
	    delete (trsfiles.tempin)
	    if (vb)
		print ("BTRES: No files found: ", ims)
	    return
	    }
	tempin = trsfiles.tempin
	tempout = trsfiles.tempout
	ims = "@" // tempout

# Remove overscan, trim, and merge amplifiers of image
# Then remove pixel to pixel variation from image (including IR fringing)
# Then extract spectra from image and add dispersion function
	tproc.gaincorr = gaincorr
	tproc.fixbias = fixbias
	tproc.fixpix = fixpix
	tproc.fixpreamp = fixpreamp
	tproc.flatten = no
	tproc.cosmic = cosmic
	tproc.threshold = ct
	tproc.sumspec = sumspec
	tproc.deltemp = dt
	tproc.saveraw = sr
	tproc.saveold = sv
	tproc.verbose = vb
	tcosmic.savebp = sv
	tproc (ims, flatten=no, extract=no, disperse=no)

	if (delim) {
	    imdelete (ims)
	    }

#  Delete temporary list files
	if (dt) {
	    delete (tempin)
	    delete (tempout)
	    }

	if (tcheck.rawdir)
	    tdata (datedir=tcheck.datedir)
end

# Feb 13 2009	New task based on FTRES
# Mar  9 2009	Return if no files found from list
# Apr 10 2009	Make default normdir = "" (working directory)
# Apr 27 2009	Read raw and working directories from CTRES
# May 12 2009	Drop avgspec parameter; add mean and median options to sumspec
# May 12 2009	Add cosmic parameter to make cosmic ray removal optional
