# TDARK1 -- Subtract dark image from TRES image, scaling for exposure
# By Doug Mink, SAO Telescope Data Center
# May 22, 2009

procedure tdark1 (image, darkfile)

string	image = ""	{prompt="Input multispec image to correct"}
string	darkfile=""	{prompt="Dark image to subtract after scaling"}

string	darkdir=""	{prompt="Directory for dark image file"}
bool	saveold = yes	{prompt="Save original file to Unflat/ (y or n)"}
bool	deltemp=yes	{prompt="Delete temporary files (yes or no"}
bool	verbose = yes	{prompt="Log progress (y or n)"}

begin
	bool vb, sv, dt, makename
	file imin, imout, imroot, imnorm, newfile, templist, ddir, dfile
	file dtemp
	int lroot
	string history, ims, tchar, flatref, imref, dpath
	real expd, expi, ratio

	imin = image
	vb = verbose
	sv = saveold
	dt = deltemp
	dfile = darkfile
	if (dfile == "")
	    makename=yes
	else
	    makename = no
	ddir = darkdir
	fixdir (ddir)
	ddir = fixdir.newdir

# Do not subtract dark field if it has already been subtracted
	imgetp.default = "0"
	imgetp (imin, "TDARK")
	if (imgetp.value != "0") {
	    if (vb) {
		print ("TDARK1: ", imin, " has been processed")
		print ("TDARK = ", imgetp.value)
		}
	    return
	    }

# Remove extension from input file, if one is present, and make output file name
	smatch (imin, ".fits")
	if (smatch.found) {
	    lroot = smatch.start - 1
	    imout = substr (imin, 1, lroot) // ".flattened.fits"
	    }
	else {
	    imout = imin // ".flattened.fits"
	    imin = imin // ".fits"
	    }

# Add binning code and FITS extension to dark file if it is missing
	if (makename) {
	    tfib1 (imin, specdir="")
	    dfile = "dark" // tfib1.binning // ".fits"
	    }
	else {
	    smatch (dfile, ".fits")
	    if (!smatch.found) {
		dfile = dfile // ".fits"
		}
	    }

# Check for the existence of the normalization spectrum locally
	if (!access (dfile)) {
	    print ("TDARK1: dark file ",dfile," not found")
	    if (ddir == "")
		ddir = "tresdata$"
	    dfile = ddir // dfile
	    }
	if (!access (dfile)) {
	    print ("TDARK1: Dark file ",dfile," not found")
	    return
	    }
	dtemp = mktemp ("tempdark") // ".fits"
	imgetp (dfile, "EXPTIME")
	expd = real (imgetp.value)
	imgetp (imin, "EXPTIME")
	expi = real (imgetp.value)
	ratio = expi / expd
	imarith (dfile, "*", ratio, dtemp)
	round (ratio, "%.5f")
	if (vb) {
	    print ("TDARK1: ", imin," -> ",imout," using ", dfile, " * ", round.value)
	    }

# Subtract scaled dark image
	imarith (imin, "-", dtemp, imout)

# Move original file to Unflat directory
	if (sv) {
	    if (!access ("Undark/")) {
		mkdir ("Udarkt")
		}
	    newfile = "Undark/" // imin
	    if (access (newfile)) {
		imdelete (newfile)
		}
	    movefiles (imin, "Undark")
	    if (vb) {
		print ("TDARK1: Original ", imin, " => Undark/")
		}
	    }
	else {
	    delete (imin)
	    }

# Rename result file to original file name
	rename (imout, imin)

# Add history line including time
	gettime (0.0, timediff=99)
	history = gettime.date // " " // gettime.time // " Dark file * " // round.value // "subtracted"
        hedit (imin, "TDARK", history, add+, verify-)
	imgetp.default = ""
	imgetp (dfile, "DARKFILE")
	dpath = imgetp.value
        hedit (imin, "TDFILE", dpath, add+, verify-)

	if (dt)
	    delete (dtemp)

end

# May 18 2009	New task based on TFLAT1
# May 22 2009	Use only binning and not aperture size for dark file name
