# TFLAT1 -- Divide TRES image by pixel normalization file from tmakeflat
#          (apflatten), saving original file in Unflat/
# By Doug Mink, SAO Telescope Data Center
# April 10, 2009

procedure tflat1 (image)

string	image = ""	{prompt="Input multispec image to correct"}
string	normfile=""	{prompt="Pixel to pixel tmakeflat normalization file"}
string	normdir=""	{prompt="Directory for normalization file"}
bool saveold = yes	{prompt="Save original file to Unflat/ (y or n)"}
bool verbose = yes	{prompt="Log progress (y or n)"}

begin
	bool vb, sv, makename
	file imin, imout, imroot, imnorm, newfile, templist, ndir, tfile
	int lroot
	string history, ims, tchar, flatref, imref

	imin = image
	vb = verbose
	sv = saveold
	imnorm = normfile
	if (imnorm == "")
	    makename=yes
	else
	    makename = no
	ndir = normdir
	fixdir (ndir)
	ndir = fixdir.newdir

# Do not flatten spectra twice
	imgetp.default = "0"
	imgetp (imin, "TFLAT")
	if (imgetp.value != "0") {
	    if (vb) {
		print ("TFLAT1: ", imin, " has been processed")
		print ("TFLAT = ", imgetp.value)
		}
	    return
	    }

# Do not try to flatten DARK or BIAS image with no fiber data
	imgetp.default = "0"
	imgetp (imin, "FIBKEY")
	if (imgetp.value == "NNN") {
	    if (vb) {
		print ("TFLAT1: ", imin, " cannot be flattened")
		}
	    return
	    }

# Remove extension from input file, if one is present, and make output file name
	smatch (imin, ".fits")
	if (smatch.found) {
	    lroot = smatch.start - 1
	    imout = substr (imin, 1, lroot) // ".flattened.fits"
	    }
	else {
	    imout = imin // ".flattened.fits"
	    imin = imin // ".fits"
	    }

# Add FITS extension to normalization file if it is missing
	if (makename) {
	    tfib1 (imin, specdir="")
	    imnorm = "flat" // tfib1.apfib // ".flat.fits"
	    }
	else {
	    smatch (imnorm, ".fits")
	    if (!smatch.found) {
		imnorm = imnorm // ".fits"
		}
	    }

# Check for the existence of the normalization spectrum locally
	if (!access (imnorm)) {
	    print ("TFLAT1: Normalization file ",imnorm," not found")
	    if (ndir == "")
		ndir = "tresdata$"
	    imnorm = ndir // imnorm
	    }
	if (!access (imnorm)) {
	    print ("TFLAT1: Normalization file ",imnorm," not found")
	    return
	    }

	if (vb) {
	    print ("TFLAT1: ", imin," -> ",imout," using ", imnorm)
	    }

# Divide out normalization spectra
	imarith (imin, "/", imnorm, imout)

# Move original file to Unflat directory
	if (sv) {
	    if (!access ("Unflat/")) {
		mkdir ("Unflat")
		}
	    newfile = "Unflat/" // imin
	    if (access (newfile)) {
		imdelete (newfile)
		}
	    movefiles (imin, "Unflat")
	    if (vb) {
		print ("TFLAT1: Original ", imin, " => Unflat/")
		}
	    }
	else {
	    delete (imin)
	    }

# Rename result file to original file name
	rename (imout, imin)

# Resolve real flat field file used for normalization
	imref = "flat" // tfib1.apfib // ".flat.fits"
	if (imnorm == imref) {
	    imgetp.default = imref
	    imgetp (imnorm, "FLATFILE")
	    tfile = imgetp.value
	    }
	else
	    tfile = imnorm

# Add history line including time
	gettime (0.0, timediff=99)
	history = gettime.date // " " // gettime.time // " pixels normalized by " // imnorm
        hedit (imin, "TFLAT", history, add+, verify-)
        hedit (imin, "TFFILE", tfile, add+, verify-)

end

# Feb 22 2008	New task separated from TFLAT
# Mar 18 2008	Fix bug: argument mismatch images/image
# May 16 2008	Add input time argument to gettime() 
# Jun  6 2008	Call TRSFIB1 instead of TRSFIB
# Aug  5 2008	Use local flattening file if possible, else tresdata$
# Aug  5 2008	Return if flattening file is not found
# Sep 25 2008	Always use local normalization file if it exists
# Dec 15 2008	Add timediff argument to GETTIME to get current time
# Dec 18 2008	Add TFFILE to give actual flattening file name

# Mar  5 2009	Fix typo in task call
# Apr 10 2009	Make default normdir = ""; search for imnorm locally first
