# File tres/Scripts/tlog.cl
# By Doug Mink
# March 19, 2009

# TLOG -- Make Starbase table of major keywords in TRES data files

procedure tlog (images)

string images="1-300"	{prompt="Files to process"}

file	imagedir=""	{prompt="Directory containing images (overrides above)"}
string	imagetype=""	{prompt="Select this image type if not null"}
string	object=""	{prompt="Select this object if not null"}
string	apfib=""	{prompt="Select fiber (s m l) binning (b) ap (1 2 3)"}
string	aperture=""	{prompt="Select on this aperture (1 2 3)"}
string	fibkey=""	{prompt="Select on this FIBKEY value"}
string	binning=""	{prompt="Select if binned this many times (1 2 null)"}
string	fibsize=""	{prompt="Select if FIBSIZ is this value"}
string	exptime=""	{prompt="Select if EXPTIME has this value"}
string	program=""	{prompt="Select objects if PROGRAM has this value"}
file	outlist="tres.log"	{prompt="File name for output list of files"}
bool	addfib=no	{prompt="Add FIBi keywords to spectra (yes or no)"}
bool	deltemp=yes	{prompt="Delete temporary files (yes or no)"}
bool	overwrite=no	{prompt="Overwrite existing list files (yes or no)"}
bool	verbose=yes	{prompt="Print table of image parameters (yes or no)"}
bool	debug=no	{prompt="Print table with more image parameters (yes or no)"}

struct *filelist

begin
    bool vb, db, dt, ow
    file imf, tempin, tempout, idir, imout
    file listfile, ofile, impath, imin
    string fkey, fib1, fib2, fib3, af, aff, afb, afs, afx
    string bin1, ims, ddir, fk, fsize, imt, ut, etimea, ra, dec
    int ns, n, nrange, lfile, maxlfile, lobj, maxlobj
    string specs, olist, lastpath, obj, prognum, prog, progname
    string fname, fpath, dateobs
    string dline1, dline2
    char tchar
    int iexp, iexp0
    real mjdmid

    vb = verbose
    db = debug
    ow = overwrite
    dt = deltemp
    ims = images

    ddir = ""
    if (imagedir != "")
	idir = imagedir
    else
	idir = trsdate.rdir

    ofile = outlist
    fk = fibkey
    afs = fibsize
    afb = binning
    aff = aperture
    imt = imagetype
    prog = program
    if (exptime != "")
	iexp = int (exptime)
    else
	iexp = 0

    if (db)
	print ("TRSLIST: Finding fiber size")

# Find fiber size, binning, and 
    af = apfib
    afx = af
    if (afx != "") {
	trsdecode (afx)
	afs = trsdecode.fibsize
	afb = trsdecode.binning
	aff = trsdecode.apertures
	}

    maxlfile = 0
    maxlobj = 0

    if (idir != "" || isrange.range)
	addfib = no

# Make list of image paths
    trsfiles.instdir = idir
    trsfiles.datedir = ddir
    trsfiles.copyfile = no
    trsfiles.verbose = no
    trsfiles.debug = no
    if (db) {
	print ("TRSLIST: About to call TRSFILES apsize=", afs, " binning=",afb, " fiber=",aff)
	print ("TRSLIST: Reading from ",ims
	}
    trsfiles (ims)
    if (trsfiles.nfiles == 0) {
	delete (trsfiles.tempin)
	if (vb)
	    print ("TLOG: No files found: ", ims)
	return
	}
    tempin = trsfiles.tempin
    tempout = trsfiles.tempout
    ims = "@" // tempout
    if (db)
	print ("TRSLIST: File list in ",tempout)

    maxlfile = trsfiles.maxlfile
    if (maxlfile < 8)
	maxlfile = 8
    maxlobj = trsfiles.maxlobj
    if (maxlobj < 6)
	maxlobj = 6

# Print heading for list of spectrum attributes
	fname = "filename"
	for (i = 9; i <= maxlfile; i = i + 1) {
	    fname = fname // " "
	    }
	dline1 = fname // "	" // "exptime	dateobs            	ra           	dec        	" //  "apfib	fibkey	program"
	fname = "-"
	for (i = 2; i <= maxlfile; i = i + 1) {
	    fname = fname // "-"
	    }
	dline2 = fname // "	-------	-------------------	----------	----------	-----	------	----"
    
    tfib1.addfib = addfib
    tfib1.verbose = no
    tfib1.debug = no

    ns = 0

# Print spectrum attributes
    olist = ""
    filelist = tempout
    while (fscan (filelist, imf) != EOF) {
	fpath = imf
	tres.filename (fpath)
	fname = tres.filename.filename
	tfib1.specdir = tres.filename.dirpath
	tfib1 (fname)
	if (tfib1.imagetype == "ERROR")
	    next
#	if (db)
#	    dpar ("tfib1")
	fkey = tfib1.fibkey
	fib1 = tfib1.fib1
	fib2 = tfib1.fib2
	fib3 = tfib1.fib3
	dateobs = tfib1.dateobs
	fsize = tfib1.fibsize
	bin1 = tfib1.bin1
	ut = substr (tfib1.dateobs,12,19)
	dateobs = tfib1.dateobs
	prognum = tfib1.prognum
	progname = tfib1.progname

# Keep only spectra which are specified
	if (af != "") {
	    if (aff == "1" && fib1 == "OFF")
		next
	    if (aff == "2" && fib2 == "OFF")
		next
	    if (aff == "3" && fib3 == "OFF")
		next
	    if (afb != bin1)
		next
	    if (afs != "" && afs != tfib1.fibsize)
		next
	    }
	if (imt != "" && tfib1.imagetype != imt) {
	    if (db)
		print ("TRSLIST: image type ",imt," != ", tfib1.imagetype)
	    next
	    }
	if (fk != "" && fkey != fk) {
	    if (db)
		print ("TRSLIST: key ",fk," != ", fkey)
	    next
	    }
	if (afs != "" && fsize != afs) {
	    if (db)
		print ("TRSLIST: fiber size ",afs," != ", fsize)
	    next
	    }
	if (afb != "" && bin1 != afb) {
	    if (db)
		print ("TRSLIST: binning ",afb," != ", bin1)
	    next
	    }
	if (object != "" && tfib1.object != object) {
		if (db)
		print ("TRSLIST: object ",tfib1.object," != ", object)
	    next
	    }
	if (iexp > 0) {
	    iexp0 = int (tfib1.exptime)
	    if (iexp0 != iexp) {
		if (debug)
		    print ("TRSLIST: ",tfib1.exptime," != ",exptime)
		next
		}
	    }
	if (prog != "") {
	    if (prognum != "") {
		if (prognum != prog) {
		    if (db)
			print ("TRSLIST: program ", prognum," != ", prog)
		    next
		    }
		}
	    else if (progname != "") {
		if (progname != prog) {
		    if (db)
			print ("TRSLIST: program ", progname," != ", prog)
		    next
		    }
		}
	    }

# Continue to process this spectrum if it passes the filters
	ns = ns + 1
	if (ns == 1 && (vb || db)) {
#	    print (idir)
	    print (dline1)
	    print (dline2)
	    if (ofile != "") {
		if (access (ofile) && ow)
		    delete (ofile)
		print (dline1, > ofile)
		print (dline2, >> ofile)
		}
	    }
	lfile = strlen (fname)
	for (i = lfile+1; i <= maxlfile; i = i + 1) {
	    fname = fname // " "
	    }
	obj = tfib1.object
	lobj = strlen (obj)
	for (i = lobj+1; i <= maxlobj; i = i + 1) {
	    obj = obj // " "
	    }
	cround (tfib1.exptime, "%.1f")
	etime = cround.value
	
	ra = tfib1.ra
	dec = tfib1.dec
	if (prognum != "")
	    prognum = prognum // " "

	if (vb)
	    printf ("%s	%s	%s	%s	%s	%s	%s	%s%s\n", fname, etime, dateobs, ra, dec, tfib1.apfib, fkey, prognum, progname)

# Save this file name in list or listfile
	if (ofile != "") {
	    if (vb)
		printf ("%s	%s	%s	%s	%s	%s	%s	%s%s\n", fname, etime, dateobs, ra, dec, tfib1.apfib, fkey, prognum, progname, >> ofile)
	    else if (ns == 1)
		print (fname, > ofile)
	    else
		print (fname, >> ofile)
	    }
	else if (ns == 1)
	    olist = fname
	else
	    olist = olist // "," // fname
	}

    if (dt) {
	if (access (tempin))
	    delete (tempin)
	if (access (tempout))
	    delete (tempout)
	}

    if (!vb && !db && olist != "")
	print (olist)
    filelist = ""

end

# Mar  9 2009	New program, based on trslist
# Mar 19 2009	Add RA and Dec; drop OBJECT; change order
