# File tres/Scripts/tpmake.cl
# By Doug Mink
# May 13, 2009

# TPMAKE -- Extract aperture spectra from a flat field or other high-flux object
#          image, divide the sky aperture by the object aperture, and make a
#          throughput correcting spectrum to be used for background subtraction.

procedure tpmake (tpspec1, tpspec2)

file tpspec1=""		{prompt="Spectrum for fiber 1"}
file tpspec2=""		{prompt="Spectrum for fiber 2"}

file	tproot="tp"	{prompt="Output file root"}
bool	verbose=yes	{prompt="Log progress (yes or no)"}
bool	debug=no	{prompt="Print output (yes or no)"}

begin

    bool vb, db
    file spec1, spec2, otpf
    string null, flat

    vb = verbose
    db = debug
    null = ""
    flat = "flat"

# Quit if input spectra do not exist
    spec1 = tpspec1
    if (!access (spec1)) {
	print ("TPMAKE: Cannot read fiber 1 reference spectrum ",spec1)
	return
	}
    spec2 = tpspec2
    if (!access (spec2)) {
	print ("TPMAKE: Cannot read fiber 2 reference spectrum ",spec2)
	return
	}

# Dispersion correct spectra if not done already
    imgetp.default = "0"
    imgetp (spec1, "TDISP")
    if (imgetp.value == "0") {
	tdisp1 (spec1)
	}
    imgetp.default = "0"
    imgetp (spec2, "TDISP")
    if (imgetp.value == "0") {
	tdisp1 (spec2)
	}

# Make throughput ratio file
    tfib1 (spec1)
    otpf = tproot // tfib1.apsize // tfib1.binning // "12.ec.fits"

# Delete existing output standard flat field file if already exists
    if (access (otpf)) {
	if (vb) {
	    print ("TPMAKE: Deleting existing ", otpf)
	    }
	delete (otpf)
	}

# Divide fiber 2 spectrum by fiber 1 spectrum for relative throughput
    if (vb) {
	print ("TPMAKE: Dividing ", spec2, " by ", spec1)
	}
    tarith (spec2, "/", spec1, otpf)
    if (vb) {
	print ("TPMAKE: Throughput file ", otpf, " made")
	}
    return

end

# Feb 13 2009	New task to make sky/object relative throughput file
# Apr 29 2009	Revised to accept spectra, not image, as input
# May 13 2009	Dispersion correct spectra if not already done
