# TPROC - Run image processing tasks on both TRES amps, combine, and flatten
# May 22, 2009
# By Doug Mink, SAO Telescope Data Center

procedure tproc (images)

string	images=""	{prompt="Images to merge"}
real	gaincorr=1.0	{prompt="Gain correction for amp2 to amp1"}
bool	fixpreamp=no	{prompt="Make preamp correction (yes or no)"}
bool	fixbias=no	{prompt="Fix bias section (yes or no)"}
bool	fixpix=yes	{prompt="Fix bad pixels (yes or no)"}
bool	subdark=no	{prompt="Subtract scaled dark file (yes or no)"}
file	darkfile=""	{prompt="Dark image for CCDPROC (null = none)"}
bool	flatten=yes	{prompt="Run tflat on each image (yes or no)"}
bool	cosmic=yes	{prompt="Remove cosmic rays from a list of images (yes or no)"}
bool	extract=yes	{prompt="Extract spectra using textract (yes or no)"}
bool	disperse=yes	{prompt="Add wavelength dispersion function to spectrum (yes or no)"}
string  normfile=""     {prompt="Pixel to pixel tmakeflat normalization file"}
string  normdir="tresdata$"     {prompt="Directory for normalization file"}
real	threshold=10.0	{prompt="Cosmic ray threshold in sigma"}
string	sumspec="no"	{prompt="Combine 2-D file as sum mean median no only(sum)"}
string	sumfile=""	{prompt="Combine 2-D file name (returned)"}
bool	rmscat=no	{prompt="Remove scattered light from image (yes or no)"}
string	aproot="flat"	{prompt="Root filename for aperature reference spectrum"}
string	apdir="tresdata$" {prompt="Directory for reference spectrum"}
bool	recenter=no	{prompt="Recenter orders during extraction (yes or no)"}
string	refroot="comp"	{prompt="Root filename for wavelength reference spectrum"}
string	refdir="tresdata$" {prompt="Directory for reference spectrum"}
string	refmode="interp" {prompt="Method (closest, interp, fit, reference)"}
bool	rmblaze=no	{prompt="Remove echelle blaze function (yes or no)"}
bool	medshift=no	{prompt="yes: median shift no: mean shift over orders"}
file	compstd=""	{prompt="File to use as ThAr shift standard (first sum mean all"}
bool	compid=yes	{prompt="Interactively ecidentify standard if yes; ecreid if no"}
bool    subsky=no       {prompt="Subtract sky spectrum (yes or no)"}
bool	deltemp=yes	{prompt="Delete intermediate image lists (yes or no)"}
bool	saveraw=yes	{prompt="Save unprocessed image files (yes or no)"}
bool	saveold=yes	{prompt="Save intermediate unprocessed image files (yes or no)"}
bool	verbose=yes	{prompt="Print process progress (yes or no)"}

struct *procfiles
begin

file	imroot, amp1, amp2, temp1, temp2, temp3, im2, imin, templist
file	istd, cstd, newtemp, dfile, tpfile
file	imfile, newfile, imlist, tref1, tref2, compdb, imin1, imin2, imin3
string	chip1, chip2, arg1, imx, ims, imsec, history, tchar, imex, rname
string	binning, nd, nf, temparg, apd, apr, mjdmid, fname
string	object, imref, imlast1, imlast2, imlast3
bool	vb, dt, sv, fp, fx, fb, flat, disp, recent, svr, sum, mshift
bool	mean, ia, id, imfirst, cid, rd, rmb, rms, dsub, subs, median, only
int	nfiles, maxlfile, maxrfile
real	gc, tshift1, tshift2, ct, shift

	vb = verbose
	dt = deltemp
	sv = saveold
	svr = saveraw
	dsub = subdark
	dfile = darkfile
	rd = yes
	subs = subsky
	fx = fixpix
	fp = fixpreamp
	fb = fixbias
	flat = flatten
	recent = recenter
	disp = disperse
	gc = gaincorr
	nf = normfile
	nd = normdir
	apr = aproot
	apd = apdir
	mshift = medshift
	imx = ".fits"
	ct = threshold
	cid = compid
	cstd = compstd
	rmb = rmblaze
	rms = rmscat

	imlast1 = ""
	imlast2 = ""
	imlast3 = ""

	ims = image
	if (ims == "") {
	    print ("TPROC: Process TRES image files to extraction")
	    print ("Usage: tproc im1,im2,...,imn")
	    print ("Usage: tproc *.fits")
	    print ("Usage: tproc @image_list_file")
	    return
	    }

# If input image is a list prepend @ if not already there
        smatch (ims, ".list")
        if (smatch.found) {
            if (substr (ims,1,1) != '@')
		ims = "@" // images
            }

	sum = no
	mean = no
	median = no
	only = no
	if (sumspec == "yes" || sumspec == "sum") {
	    sum = yes
	    }
	else if (sumspec == "only") {
	    only = yes
	    sum = yes
	    }
	else if (sumspec == "mean") {
	    mean = yes
	    }
	else if (sumspec == "median") {
	    median = yes
	    }
	if (cstd == "sum") {
	    sum = yes
	    }
	else if (cstd == "mean") {
	    mean = yes
	    }
	else if (cstd == "median") {
	    median = yes
	    }

	if (!defpac ("mscred"))
	    mscred

#  Make temporary list of input files
	tchar = substr (ims,1,1)
	templist = mktemp ("tempproc") // ".list"
	files (ims, >templist)

# Read through input files
	procfiles = templist
	nfiles = 0
        while (fscan (procfiles,imin) != EOF) {

# Process files as images
	    tproc1 (imin,fixpreamp=fp,fixbias=fb,fixpix=no,deltemp=dt,saveraw=svr,saveold=sv,verbose=vb)

# Correct gain differences between amplifiers and merge them
	    tgain1 (imin,g2=gc,deltemp=dt,saveold=sv,verbose=vb)

# Subtract dark image, if requested
	    if (dsub)
		tdark1 (imin, dfile, saveold=sv, verbose=vb)

# Fix bad pixels, if requested
	    if (fixpix) {
		fixpix.linterp = "INDEF"
		fixpix.cinterp = "INDEF"
		fixpix.verbose = vb
		fixpix.pixels = vb
		fixpix (imin, "tresdata$badpix")
		}

# Flatfield correct spectra
	    if (flat)
		tflat1 (imin,normfile=nf,normdir=nd,saveold=sv,verbose=vb)

# Add barycentric velocity correction (BCV) to image header
	    tsetbcv (imin)

	    nfiles = nfiles + 1
	    }

# Remove cosmic rays, if requested
	if (cosmic) {
	    if (nfiles > 1) {
		temparg = "@" // templist
		tcosmic (temparg, threshold=ct, savebp=sv, saveold=sv, deltemp=dt, verbose=vb)
		}
	    else if (vb) {
		print ("TPROC: Cannot remove cosmic rays from a single file")
		}
	    }

# Compute sum of images if requested, and add to extraction list
	sumfile = ""
	if (sum) {
	    temparg = "@" // templist
	    trssum (temparg, outroot="")
	    sumfile = trssum.outfile
	    if (trssum.nfiles > 1) {
		if (only) {
		    newtemp = mktemp ("tempsum") // ".list"
		    print (sumfile, >newtemp)
		    delete (templist)
		    rename (newtemp, templist)
		    }
		else if (cstd == "sum") {
		    newtemp = mktemp ("tempsum") // ".list"
		    print (sumfile, >newtemp)
		    cat (templist, >>newtemp)
		    delete (templist)
		    rename (newtemp, templist)
		    cstd = "first"
		    }
		else {
		    print (sumfile, >>templist)
		    }
		}
	    }

# Compute mean of images if requested, and add to extraction list
	if (mean) {
	    temparg = "@" // templist
	    trsavg (temparg, outroot="")
	    sumfile = trsavg.outfile
	    if (trsavg.nfiles > 1) {
		if (only) {
		    newtemp = mktemp ("tempmean") // ".list"
		    print (sumfile, >newtemp)
		    delete (templist)
		    rename (newtemp, templist)
		    }
		else if (cstd == "mean") {
		    newtemp = mktemp ("tempavg") // ".list"
		    print (sumfile, >newtemp)
		    cat (templist, >>newtemp)
		    delete (templist)
		    rename (newtemp, templist)
		    cstd = "first"
		    }
		else
		    print (sumfile, >>templist)
		}
	    }

# Compute median of images if requested, and add to extraction list
	if (median) {
	    temparg = "@" // templist
	    trsmed (temparg, outroot="")
	    sumfile = trsmed.outfile
	    if (trsmed.nfiles > 1) {
		if (only) {
		    newtemp = mktemp ("tempmed") // ".list"
		    print (sumfile, >newtemp)
		    delete (templist)
		    rename (newtemp, templist)
		    }
		else if (cstd == "median") {
		    newtemp = mktemp ("tempmed") // ".list"
		    print (sumfile, >newtemp)
		    cat (templist, >>newtemp)
		    delete (templist)
		    rename (newtemp, templist)
		    cstd = "first"
		    }
		else
		    print (sumfile, >>templist)
		}
	    }

# Extract 1D x orders spectrum and add dispersion, if requested
	if (extract) {
	    procfiles = templist
	    tshift1 = 0.0
	    tshift2 = 0.0
	    imfirst = yes
            while (fscan (procfiles,imin) != EOF) {

# Optionally remove background before extracting spectra
		if (rms) {
		    tscat1 (imin, saveold=sv, deltemp=dt, verbose=vb)
		    }

# Extract however many spectra are in this image
		textract1 (imin, aproot=apr, apdir=apd, redo=rd, recenter=recent, verbose=vb)

# Add wavelength dispersion, if requested
		if (disp) {
		    tfib1 (imin)
		    object = tfib1.object
		    imin1 = textract1.spec1
		    imin2 = textract1.spec2
		    imin3 = textract1.spec3

# Make throughput correction spectrum if it will be needed
		    if (subs) {
			flat1 = aproot // tfib1.apsize // tfib1.binning // "1.ec.fits"
			flat2 = aproot // tfib1.apsize // tfib1.binning // "2.ec.fits"
			tpfile = tpmake.tproot // tfib1.apsize // tfib1.binning // "12.ec.fits"
			if (!access (tpfile))
			    tpmake (flat1, flat2, reuse=yes)
			}

# Add wavelength dispersion to fiber 3 spectrum
		    if (imin3 != "") {
			if (rmb) {
			    tblaze1 (imin3, saveold=sv, verbose=vb)
			    }
			imgetp.default = "NONE"
			imgetp (imin3, "FIB3")
			if (imgetp.value == "COMP") {
			    tcal1.shift = 0.0
			    tcal1.saveold = sv
			    tcal1.deltemp = dt
			    tcal1.verbose = vb
			    tcal1.redo = rd
			    ia = no
			    id = no
			    istd = ""
			    if (cstd == "first") {
				if (imfirst) {
				    if (cid)
					ia = yes
				    else
					id = yes
				    }
				}
			    else if (cstd == "all") {
				if (!imfirst) {
				    istd = imlast3
				    }
				if (cid)
				    ia = yes
				else
				    id = yes
				}
			    else if (imin == cstd) {
				if (cid)
				    ia = yes
				else
				    id = yes
				}
			    else {
				istd = cstd
				}
			    tcal1 (imin3, imref=istd, reidentify=id, medshift=mshift, interactive=ia, disperse=disp)
			    if (ia || id) {
				tharset (imin3)
				imref = tharset.comppath
				shift = 0.0
				}
			    else {
				imref = tcal1.imref
				shift = tcal1.shift
				}
			    tshift1 = shift
			    tshift2 = shift
			    print ("TPROC: simultaneous ThAr not working yet")

			# Add this shift to the shift database for this apfib
			    tharadd (imin3, imref, shift)
			    imlast3 = imin3
			    }
			else 
			    tdisp1 (imin3, refselect=refmode, refroot=refroot, refdir=refdir, saveold=sv, verbose=vb)
			imlast3 = imin3
			}

# Compute pixel shifts for apertures 1 and 2 from existing ThAr spectra
		    else if (tfib1.imagetype != "COMP") {
			if (imin1 != "") {
			    tharshift1 (imin1, refroot=refroot, refselect=refmode, verbose=vb)
			    tref1 = tharshift1.refpath
			    tshift1 = tharshift1.shift
			    }
			if (imin2 != "") {
			    tharshift1 (imin2, refroot=refroot, refselect=refmode, verbose=vb)
			    tref2 = tharshift1.refpath
			    tshift2 = tharshift1.shift
			    }
			}

# Add wavelength dispersion to fiber 1 spectrum
		    if (imin1 != "") {
			if (rmb) {
			    tblaze1 (imin1, saveold=sv, verbose=vb)
			    }
			imgetp.default = "NONE"
			imgetp (imin1, "FIB1")

# Compute shift and fit dispersion for ThAr spectrum
			if (imgetp.value == "COMP") {
			    tcal1.shift = 0.0
			    tcal1.saveold = sv
			    tcal1.deltemp = dt
			    tcal1.verbose = vb
			    tcal1.redo = rd
			    ia = no
			    id = no
			    istd = ""
			    if (cstd == "first") {
				if (imfirst) {
				    if (cid)
					ia = yes
				    else
					id = yes
				    }
				}
			    else if (cstd == "all") {
				if (!imfirst) {
				    istd = imlast1
				    }
				if (cid)
				    ia = yes
				else
				    id = yes
				}
			    else if (imin == cstd) {
				if (cid)
				    ia = yes
				else
				    id = yes
				}
			    else {
				istd = cstd
			    }
			    tcal1 (imin1, imref=istd, reidentify=id, medshift=mshift, interactive=ia, disperse=disp)
			    if (ia || id) {
				tharset (imin1)
				imref = tharset.comppath
				shift = 0.0
				}
			    else {
				imref = tcal1.imref
				shift = tcal1.shift
				}
			    tshift1 = shift

			# Add this shift to the shift database for this apfib
			    tharadd (imin1, imref, shift)
			    imlast1 = imin1
			    }
#			else if (imin3 != "")
#			    tdisp1 (imin1, shift=tshift1, refpath=tref1, refdir=refdir, refroot=refroot, saveold=sv, verbose=vb)
			else
			    tdisp1 (imin1, refpath="today", refselect=refmode, refdir=refdir, refroot=refroot, saveold=sv, verbose=vb)
			}

# Add wavelength dispersion to fiber 2 spectrum
		    if (imin2 != "") {
			if (rmb) {
			    tblaze1 (imin2, saveold=sv, verbose=vb)
			    }
			imgetp.default = "NONE"
			imgetp (imin2, "FIB2")

		# Compute shift and fit dispersion for ThAr spectrum
			if (imgetp.value == "COMP") {
			    tcal1.shift = 0.0
			    tcal1.saveold = sv
			    tcal1.deltemp = dt
			    tcal1.verbose = vb
			    tcal1.redo = rd
			    id = no
			    ia = no
			    istd = ""
			    if (cstd == "first") {
				if (imfirst) {
				    if (cid)
					ia = yes
				    else
					id = yes
				    }
				}
			    else if (cstd == "all") {
				if (!imfirst) {
				    istd = imlast2
				    }
				if (cid)
				    ia = yes
				else
				    id = yes
				}
			    else if (imin == cstd) {
				if (cid)
				    ia = yes
				else
				    id = yes
				}
			    else {
				istd = cstd
				}
			    tcal1 (imin2, imref=istd, reidentify=id, medshift=mshift, interactive=ia, disperse=disp)
			    if (ia || id) {
				tharset (imin2)
				imref = tharset.comppath
				istd = imref
				shift = 0.0
				}
			    else {
				imref = tcal1.imref
				shift = tcal1.shift
				istd = imref
				}
			    tshift2 = shift

			# Add this shift to the shift database for this apfib
			    tharadd (imin2, imref, shift)
			    imlast2 = imin2
			    }
#			else if (imin3 != "")
#			    tdisp1 (imin2, shift=tshift2, refpath=tref2, refdir=refdir, refroot=refroot, saveold=sv, verbose=vb)
			else
			    tdisp1 (imin2, refpath="today", refselect=refmode, refdir=refdir, refroot=refroot, saveold=sv, verbose=vb)
			}
		    }
		imfirst = no
		}
	    }

	if (dt)
	    delete (templist)
	procfiles = ""
end

# Jan 09 2008	New program to process list of TRES images
# Jan 17 2008	Update normfile and normdir prompts
# Jan 24 2008	Add optional bad pixel file correction with fixpix parameter
# Jan 28 2008	Add optional cosmic ray removal from list of images with TCOSMIC
# Jan 31 2008	Add optional spectrum extraction from list of images
# Feb  8 2008	Add saveraw to save raw images but not others
# Feb  8 2008	Add optional wavelength dispersion using TDISP
# Feb 11 2008	Change temporary list file from temproclist* to tempproc*list
# Feb 12 2008	Add recenter parameter for extraction
# Feb 25 2008	Use TEXTRACT1 to process images one at a time, not TEXTRACT
# Feb 25 2008	Use TPROC1 to process images one at a time, not TPREPROC
# Feb 25 2008	Call TPREAMP1 and TGAIN1 instead of using TPREPROC
# Feb 25 2008	Change name of TDISP to TDISP1 for consistency
# Mar 18 2008	Drop deltemp from arguments to TFLAT1
# Apr 10 2008	Drop deltemp from arguments to TDISP1
# Mar 18 2008	Add TAMPMERGE
# Mar 19 2008	Run TCAL1 on COMP spectra instead of TDISP1
# Mar 26 2008	Pass saveraw to TPROC1
# Mar 31 2008	Set dispersion pixel shift from most recent calibration lamp
# Apr  2 2008	Add deltemp, saveold, and verbose arguments to tcosmic and tcal1
# Apr  2 2008	Add deltemp and verbose arguments to textract1 and tdisp1
# Apr  4 2008	Add sumspec parameter to process summed spectra
# Apr 30 2008	Move optional preamp fix to TPROC1 task
# May 22 2008	Drop unused refshift parameter
# May 23 2008	Set dispersion reference spectrum using THARSHIFT1 task
# Jul 17 2008	Save reference, time and shift of ThAr spectra in database files
# Jul 18 2008	Add refmode parameter; initialize trssum.outroot=""
# Jul 31 2008	Add mean option to sumspec parameter
# Aug 15 2008	Pass disperse parameter to TCAL1
# Sep 19 2008	Add medshift parameter to allow option of tcal1 median not mean
# Sep 19 2008	Pass refmode to disp1 refselect
# Sep 19 2008	Do not use fiber 3 shift for fibers 1 and 2
# Oct 15 2008	Add threshold parameter to set cosmic ray threshold
# Oct 16 2008	Add compstd parameter for file name to become comp*ec.fits
# Nov 14 2008	Add option to fit first calibration spectrum interactively
# Dec 10 2008	If only one file, do not try to process a summed file
# Dec 11 2008	If only one file, do not try to process an averaged file
# Dec 17 2008	Round output shift to 5 places and MJD to 6 places
# Dec 17 2008	Resolve reference spectrum to specific date and number
# Dec 18 2008	Print headers on shift tables
# Dec 18 2008	Reset ThAr path and shift for interactively fit spectra
# Dec 30 2008	Set tcal1.reid=yes if compstd is set
# Dec 31 2008	Assume shift set to 0 in TCAL1 if reidentify or interactive set
# Dec 31 2008	Make @ optional if image is *.list file

# Feb 13 2009	Add avgspec to allow mean and/or summed image output
# Feb 19 2009	Add "all" option to compstd parameter to always refit COMPs
# Feb 19 2009	Add imini and imlasti parameters to simplify processing
# Mar  5 2009	Fix typo in object processing
# Mar  6 2009	Add compid to allow interacive line ID
# Mar 10 2009	Initialize imref in TCAL1 to null string
# Mar 11 2009	Add mean and sum options to compstd
# Mar 11 2009	Use THARADD to make file of COMP shifts
# Mar 16 2009	Fix bad initialization of id for fiber 2
# Mar 16 2009	Always redo extraction and wavelength calibration
# Mar 17 2009	Clean up code to process COMPs correctly
# Apr 13 2009	Add separate call to FIXPIX on merged image file
# Apr 23 2009	Add parameter rmscat and optionally remove scattered light (2d)
# Apr 23 2009	Add parameter rmblaze and optionally remove echelle blaze function (1d)
# Apr 27 2009	Add darkfile parameter to be passed to TPROC1 and CCDPROC
# Apr 28 2009	Pass darkfile parameter to TDARK1 if new parameter subdark=yes
# Apr 30 2009	Pass deltemp, saveold, and verbose to TDARK, TSCAT1, and TBLAZE1
# May 12 2009	Add mean, median options to sumspec parameter; drop avgspec
# May 12 2009	Add sumfile to return name of combined output file, if any
# May 13 2009	Pass medshift parameter to TCAL1, fixing bug
# May 22 2009	Add sum option to sumspec parameter, same as yes
