# File tres/Scripts/trsgroup.cl
# April 13, 2009
# By Doug Mink

# TRSGROUP: Make lists of files to process

procedure trsgroup (images)

string	images="1-300"	{prompt="Files to process"}

string	datedir=""	{prompt="Date for images"}
file	instdir=""	{prompt="Directory containing date directories"}
file	imagedir=""	{prompt="Directory containing images (overrides above)"}
file	datafile="data.db"	{prompt="File for observation database"}
file	biaslist="bias.list"	{prompt="File listing groups of bias images"}
file	darklist="dark.list"	{prompt="File listing groups of dark images"}
file	flatlist="flat.list"	{prompt="File listing groups of flat fields"}
file	complist="comp.list"	{prompt="File listing groups of ThAr exposures"}
file	objlist="obj.list"	{prompt="File listing groups of observations"}
string	program=""	{prompt="Program number of objects to keep (null for all"}
bool	redo=yes	{prompt="If yes, start from scratch even if already run"}
bool	verbose=yes	{prompt="Summarize results (yes or no)"}
bool	debug=no	{prompt="Print file sorting instead of writing files (yes or no)"}

struct *datalist

begin
    file listname
    file imf, compname
    string imt, obj, mjd, apfib, fibkey, obj0, apfib0, progs, prog
    string fib1, fib2, fib3, sexp
    int ifile, ibias, idark, iexp, ngroups, iobj, icomp, iflat, lfk
    real exp, comp, exp0, comp0
    bool savespec, vb, db

    vb = verbose
    db = debug

    biaslist = trsproc.biaslist
    darklist = trsproc.darklist
    flatlist = trsproc.flatlist
    complist = trsproc.complist
    objlist = trsproc.objlist

    if (redo && access (datafile)) {
	if (vb)
	    print ("TRSGROUP: Deleting existing database file ",datafile)
	delete (datafile)
	if (vb)
	    print ("TRSGROUP: Deleting existing .list files")
	delete ("*.list")
	}

# List all of the data files for this night
    if (access (datafile)) {
	if (vb)
	    print ("TRSGROUP: Reading database from existing file ",datafile)
	}
    else {
	trslist (images, datedir=datedir, instdir=instdir, imagedir=imagedir,
		 imagetype="", object="",apfib="",aperture="",fibkey="",
		 binning="",fibsize="",exptime="",outlist=datafile,
		 program=program,addfib=no,deltemp=yes,overwrite=no,
		 verbose=yes,debug=no)
	}

# Write list files and master list of list files for processing
    obj0 = ""
    apfib0 =""
    exp0 = 0.0
    ifile = 0
    ibias = 0
    idark = 0
    iflat = 0
    icomp = 0
    iobj = 0
    savespec = no
    listname = ""
    datalist = datafile
    while (fscan (datalist,imf,imt,obj,mjd,apfib,fibkey,exp,comp,prog) != EOF) {
	if (imf == "filename")
	    next
	if (substr (imf,1,4) == "----")
	    next
#	print ("TRSGROUP: File ", imf)

# Find the configuration file into which this image should be listed
	iexp = int (exp)
	if (iexp == 0) {
	    round (exp,"%.1f")
	    sexp = round.value
	    }
	else
	    sexp = iexp
	fib1 = substr (fibkey,1,1)
	fib2 = substr (fibkey,2,2)
	lfk = strlen (fibkey)
	if (lfk < 3)
	    fib3 = ""
	else
	    fib3 = substr (fibkey,3,3)
	if (fib3 == "N" || fib3 == "") {
	    listname = obj // apfib// fib1 // fib2 // "x" // sexp // ".list"
	    }
	else
	    listname = obj // apfib // "x" // sexp // ".list"
	compname = obj // apfib // ".list"
	savespec = no

# Add to existing list
	if (access (listname)) {
	    if (vb)
		print ("TRSGROUP: Adding ",imf," to existing object list file ",listname)
	    if (db)
		print (imf, " >> ",listname)
	    else {
		print (imf, >>listname)
		if (complist != "" && (obj == "COMP" || obj == "comp")) {
		    if (access (compname))
			print (imf, >>compname)
		    else
			print (imf, >compname)
		    }
		}
	    }

# Create new list with this image
	else {

# Add bias list name to bias list
	    if (biaslist != "" && (obj=="BIAS" || obj == "bias")) {
		savespec = yes
		if (db) {
		    if (ibias == 0)
			print (listname, " > ", biaslist)
		    else
			print (listname, " >> ", biaslist)
		    }
		else {
		    if (ibias == 0)
			print (listname, >biaslist)
		    else
			print (listname, >>biaslist)
		    }
		ibias = ibias + 1
		}

# Add dark list name to dark list
	    else if (darklist != "" && (obj=="DARK" || obj == "dark")) {
		savespec = yes
		if (db) {
		    if (idark == 0)
			print (listname, " > ", darklist)
		    else
			print (listname, " >> ", darklist)
		    }
		else {
		    if (idark == 0)
			print (listname, >darklist)
		    else
			print (listname, >>darklist)
		    }
		idark = idark + 1
		}

# Add ThAr list names to comp list
	    else if (complist != "" && (obj == "COMP" || obj == "comp")) {
		savespec = yes
		if (db) {
		    if (icomp == 0)
			print (listname, " > ", complist)
		    else
			print (listname, " >> ", complist)
		    }
		else {
		    if (icomp == 0)
			print (listname, >complist)
		    else
			print (listname, >>complist)
		    }
		icomp = icomp + 1
		}

# Add flat field list name to flat list
	    else if (flatlist != "" && (obj == "FLAT" || obj == "flat")) {
		savespec = yes
		if (db) {
		    if (iflat == 0)
			print (listname, " > ", flatlist)
		    else
			print (listname, " >> ", flatlist)
		    }
		else {
		    if (iflat == 0)
			print (listname, >flatlist)
		    else
			print (listname, >>flatlist)
		    }
		iflat = iflat + 1
		}

# Add object configuration list name to object list
	    else if (objlist != "") {
		savespec = yes
		if (db) {
		    if (iobj == 0)
			print (listname, " > ", objlist)
		    else
			print (listname, " >> ", objlist)
		    }
		else {
		    if (iobj == 0)
			print (listname, >objlist)
		    else
			print (listname, >>objlist)
		    }
		iobj = iobj + 1
		}

	    else if (vb) {
		print ("TRSGROUP: Not saving ",listname)
		}

# Save this image file name to the new configuration list file
	    if (db)
		print (imf, " > ", listname)
	    else if (savespec)
		print (imf, >listname)

# Save a COMP image name to the appropriate ThAr list file, too
	    if (complist != "" && (obj == "COMP" || obj == "comp")) {
		if (access (compname))
		    print (imf, >>compname)
		else if (savespec)
		    print (imf, >compname)
		}
	    }

	ifile = ifile + 1
	exp0 = exp
	apfib0 = apfib
	obj0 = obj
	}

# Summarize the sorting process
    ngroups = iobj + ibias + idark + iflat + icomp
    if (vb)
	print ("TRSGROUP: ", ifile, "files split into ", ngroups, " groups")

    datalist = ""
end
# Jul 16 2008	New task
# Jul 30 2008	Make additional lists of COMP files by configuration only
# Aug 15 2008	Separate dark and bias image lists; list names are in TRSPROC
# Sep  9 2008	Fix bug so that all ThAr files are saved to exp-free lists
# Nov 12 2008	Check for lower case bias, dark, flat, and comp, too
# Nov 13 2008	Check for specified programs (program from TFIB1)
# Nov 14 2008	Read in starbase table with column headings
# Nov 17 2008	Work with unsorted data file

# Mar 10 2009	Fix bug which put bias images on object list
# Mar 12 2009	Add redo parameter to start from scratch every time
# Apr 10 2009	Add FIBKEY codes to list file names
# Apr 13 2009	Fix bug setting fib 3 when adjusted for 2-fiber mode
