# TXDELETE - Delete local copies of TRES data files from a list
# March 11, 2009
# By Doug Mink, SAO Telescope Data Center

procedure txdelete (images)

string	images=""	{prompt="Images to check"}

bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}
bool	debug=no	{prompt="Print more file information (y or n)"}

begin

file	templist, imin
string	ims, ofile, froot
bool	vb, db

	vb = verbose
	ims = images
	db = debug

	if (ims == "") {
	    print ("TXDELETE:  Delete local copies of TRES data files")
	    print ("Usage: txdelete <data immges and spectra>")
	    return
	    }

# If input image is a list prepend @ if not already there
	smatch (ims, ".list")
	if (smatch.found) {
	    if (substr (ims,1,1) != '@') {
		if (!access (ims)) {
		    print ("TXDELETE: List file ",ims," not found")
		    return
		    }
		ims = "@" // ims
		}
	    }

# Make sure that we are in a working directory, not a raw data directory
	tcheck

#  Make temporary list of input files
	trsfiles (ims, instdir=".", datedir="", verbose=vb, debug=db, copy-)
	if (trsfiles.nfiles == 0) {
	    delete (trsfiles.tempin)
	    if (vb)
		print ("TXDELETE: No files found: ", ims)
	    return
	    }
	templist = trsfiles.tempout
	list = templist
        while (fscan (list,imin) != EOF) {

# Delete image
	    delete (imin)

# Delete image derivatives
	    filename (imin)
	    fileroot (filename.filename)
	    froot = fileroot.rootname
	    ofile = froot // "*.fits"
	    delete (ofile)

# Delete database aperture file
	    ofile = "database/ap" // froot
	    if (db)
		print ("TXDELETE: Trying to delete ", ofile)
	    delete (ofile)

# Delete database dispersion file
	    ofile = "database/ec" // froot // "_f*.ec"
	    if (db)
		print ("TXDELETE: Trying to delete ", ofile)
	    delete (ofile)
	    }

	if (deltemp) {
	    delete (templist)
	    delete (trsfiles.tempin)
	    delete (trsfiles.tempout)
	    }
end

# Mar 11 2009	New task to delete TRES data images by file or sequence num.
