# File tres/src/t_tarith.x
# March 6, 2009
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# Copyright(c) 2009 Smithsonian Astrophysical Observatory
# You may do anything you like with this file except remove this copyright.
# The Smithsonian Astrophysical Observatory makes no representations about
# the suitability of this software for any purpose.  It is provided "as is"
# without express or implied warranty.
 
# TARITH is an IRAF task for operating on two spectra with any disersion function
# For arguments, see parameter file tarith.par.
# Information is shared in common blocks defined in "rvsao.com".
 
include	<imhdr.h>
include	<imio.h>
include	<fset.h>

# operation defs
define OP_ADD		1
define OP_SUB		2
define OP_MUL		3
define OP_DIV		4
define OP_FUNCTIONS	"|+|-|*|/|"

procedure t_tarith ()

int	i
char	spec1[SZ_PATHNAME]	# First spectrum file name
char	sdir1[SZ_PATHNAME]	# First spectra directory
char	spath1[SZ_PATHNAME]	# First spectrum path name
char	sname1[SZ_LINE]
pointer spix1			# First spectrum
pointer spim1			# First spectrum image header
pointer spsh1			# First spectrum header
int	npix1			# Number of pixels in first spectrum
char	spec2[SZ_PATHNAME]	# Second spectrum file name
char	sdir2[SZ_PATHNAME]	# Second spectra directory
char	spath2[SZ_PATHNAME]	# Second spectrum path name
char	sname2[SZ_LINE]
pointer spix2			# Second spectrum
pointer spim2			# Second spectrum image header
pointer spsh2			# Second spectrum header
int	npix2			# Number of pixels in second spectrum
char	resfile[SZ_PATHNAME]	# Output spectrum file name
char	resdir[SZ_PATHNAME]	# Output spectrum directory
char	respath[SZ_PATHNAME]	# Output spectrum pathname
int	logfiles		# List of log files
char	logfile[SZ_PATHNAME]	# Log file name
char	tstring[SZ_LINE]
char	resname[SZ_LINE]
char	taskname[SZ_LINE]
char	operation[4]
int	op
int	mode
int	nlogfd
int	logfd[10]

int	order, norder1, norder2

pointer	resspec		# Summed spectrum
char	str[SZ_LINE]
int	fd
int	ldir
pointer imres		# Output spectrum image header structure
pointer shres		# Output spectrum header structure
int	lres
int	npix
bool	speccur
 
bool	clgetb()
int	clgwrd(), clpopnu(), clgeti(), clgfil(), open()
int	strlen()
real	clgetr()

include "tresop.com"

begin
	nlogfd = 0
	imres = NULL
	shres = NULL
	npix = 0
	mode = READ_ONLY
	npix01 = 0
	npix02 = 0
	wl1 = NULL
	wl2 = NULL
	spix2r = NULL

# Get task parameters.

# Print processing information
	debug  = clgetb ("debug")

# First spectrum to operate on
	call clgstr ("spectrum1",spec1,SZ_PATHNAME)
	call clgstr ("specdir1",sdir1,SZ_PATHNAME)

# Open first spectrum
	ldir = strlen (sdir1)
	if (ldir > 0) {
	    if (sdir1[ldir] != '/') {
		sdir1[ldir+1] = '/'
		sdir1[ldir+2] = EOS
		}
	    call strcpy (sdir1,spath1,SZ_PATHNAME)
	    call strcat (spec1,spath1,SZ_PATHNAME)
	    }
	else
	    call strcpy (spec1,spath1,SZ_PATHNAME)
	call gettres (spath1, 1, spix1, spim1, spsh1, norder1,
		      npix1, sname1, mode, debug)
	if (debug) {
	    call printf ("TARITH: First file is %s[1-%d]\n")
		call pargstr (spath1)
		call pargi (norder1)
	    call flush (STDOUT)
	    }

# Second spectrum to operate on
	call clgstr ("spectrum2",spec2,SZ_PATHNAME)
	call clgstr ("specdir2",sdir2,SZ_PATHNAME)

# Open second spectrum
	ldir = strlen (sdir2)
	if (ldir > 0) {
	    if (sdir2[ldir] != '/') {
		sdir2[ldir+1] = '/'
		sdir2[ldir+2] = EOS
		}
	    call strcpy (sdir2,spath2,SZ_PATHNAME)
	    call strcat (spec2,spath2,SZ_PATHNAME)
	    }
	else
	    call strcpy (spec2,spath2,SZ_PATHNAME)
	call gettres (spath2, 1, spix2, spim2, spsh2, norder2,
		      npix2, sname2, mode, debug)
	IM_UPDATE(spim2) = NO
	call closetres (spim2, spsh2)
	if (debug) {
	    call printf ("TARITH: Second file is %s[1-%d]\n")
		call pargstr (spath2)
		call pargi (norder2)
	    call flush (STDOUT)
	    }

# Print an error message and exit if not same number of orders
	if (norder1 != norder2) {
	    call printf ("TARITH: Order mismatch %d != %d\n")
		call pargi (norder1)
		call pargi (norder2)
	    call flush (STDOUT)
	    return
	    }

# Operation on spectra
	op = clgwrd ("operation",operation, 4, OP_FUNCTIONS)
	if (debug) {
	    call printf ("TARITH: Operation is %s = %d\n")
		call pargstr (operation)
		call pargi (op)
	    call flush (STDOUT)
	    }

# Plot input spectra
	pltspec = clgetb ("spec_plot")

# Interact with display of input spectra
	intspec = clgetb ("spec_int")

# Number of times to smooth (1-2-1) input data plots
	ssmooth = clgeti ("ssmooth")

# Minimum and maximum values for data in graphs
	ymin = clgetr ("ymin")
	ymax = clgetr ("ymax")

# Plot output spectrum
	pltres = clgetb ("res_plot")

# Number of times to smooth (1-2-1) output spectrum
	rsmooth = clgeti ("rsmooth")

# Interact with display of final composite spectrum
	intres = clgetb ("res_int")

# Interact with display of input spectra
	speccur = clgetb ("spec_int")

# Load fitting parameters
	call getfitpars()

# Open log files and write a header.
	logfiles = clpopnu ("logfiles")
	call fseti (STDOUT, F_FLUSHNL, YES)
	i = 0
	call strcpy ("tres.tarith",taskname,SZ_LINE)
	while (clgfil (logfiles, logfile, SZ_PATHNAME) != EOF) {
	    fd = open (logfile, APPEND, TEXT_FILE)
	    if (fd == ERR) break
	    call loghead (taskname,str)
	    call fprintf (fd, "%s\n")
		call pargstr (str)
	    i = i + 1
	    logfd[i] = fd
	    }
	nlogfd = i
	call clpcls (logfiles)

# File to which to write summed spectrum
	call clgstr ("resspec",resfile,SZ_PATHNAME)
	call clgstr ("resdir",resdir,SZ_PATHNAME)
	call clgstr ("resname",resname,SZ_PATHNAME)

# Set up path for output file
	lres = strlen (resdir)
	if (lres == 0) { 
	    call strcpy (resfile, respath, SZ_PATHNAME)
	    call mkfits (respath)
	    }
	else {
	    if (resdir[lres] != '/') {
		resdir[lres+1] = '/'
		resdir[lres+1] = EOS
		}
	    call strcpy (resdir, respath, SZ_PATHNAME)
	    call strcat (resfile, respath, SZ_PATHNAME)
	    call mkfits (respath)
	    }
	if (strlen (resname) < 1) {
	    call strcpy (sname1, resname, SZ_IMTITLE)
	    call strcat (" ", resname, SZ_IMTITLE)
	    call strcat (operation, resname, SZ_IMTITLE)
	    call strcat (" ", resname, SZ_IMTITLE)
	    call strcat (sname2, resname, SZ_IMTITLE)
	    }

#  Open the output file
	call res_open (imres, respath, resspec, npix1, norder1, spim1)
	if (imres == ERR) {
	    call eprintf ("OPSPEC: Error writing composite spectrum %s\n")
		call pargstr (respath)
	    return
	    }
	else if (debug) {
	    call printf ("TARITH: Output to %s\n")
		call pargstr (respath)
	    call flush (STDOUT)
	    }
	call res_init (spim1, imres, spsh1, shres, resname)
	IM_UPDATE(spim1) = NO
	call closetres (spim1, spsh1)

	if (debug) {
	    call printf ("TARITH: %s initialized\n")
		call pargstr (respath)
	    call flush (STDOUT)
	    }

# Operate separately on each order of the spectra
	do order = 1, norder1 {
	    if (debug) {
		call printf ("TARITH: %s[%d] %s %s[%d]\n")
		    call pargstr (spec1)
		    call pargi (order)
		    call pargstr (operation)
		    call pargi (order)
		    call pargstr (spec2)
		}
	    call opspec (spec1, spath1, spec2, spath2, order, norder1, op,
                  imres, respath, resname, resspec)
	    }

# Add this operation to the result spectrum's header
	call sprintf (tstring,SZ_LINE,"tres.tarith: %s %s %s")
	    call pargstr (spec1)
	    call pargstr (operation)
	    call pargstr (spec2)
	call imputh (imres, "HISTORY", tstring)

# Close the log files
	if (nlogfd > 0) {
	    do i = 1, nlogfd {
		call close (logfd[i])
		}                                              
	    }

#  Close the output composite spectrum file
	if (imres != NULL)
	    call res_close (imres, shres, resspec, debug)

	if (wl1 != NULL)
	    call mfree (wl1, TY_REAL)
	if (wl2 != NULL)
	    call mfree (wl2, TY_REAL)
	if (spix2r != NULL)
	    call mfree (spix2r, TY_REAL)

end

# Feb  9 2009	New task based on rvsao/Sumspec/t_sumspec.x
# Mar 16 2009	Drop unused variables
