# File t_tcoslim.x
# November 8, 2007
# By Doug Mink after Nelson Caldwell

include <error.h>
include <ctype.h>
include <imhdr.h>
include <imio.h>
include <mach.h>

define	MAXCOL		4096
define	MAXLINE		4700

# TCOSLIM -- Make limit file from median file

procedure t_tcoslim ()

pointer	im1, im2, linein, lineout
real	datalim[MAXCOL,MAXLINE]
char	medfile[SZ_FNAME]
char	limfile[SZ_FNAME]
long	v[IM_MAXDIM]
bool	verbose, debug
int	i, j, nc, nl
real	datai, vmin, datai0, threshold
pointer	imgl2r(), immap()
real	clgetr()
int	impnlr()
bool	clgetb()
long	clktime()

begin

# Get parameters
	call clgstr ("medfile", medfile, SZ_FNAME)
	call clgstr ("limfile", limfile, SZ_FNAME)
	vmin = clgetr ("vmin")
	threshold = clgetr ("threshold")
	verbose = clgetb ("verbose")
	debug = clgetb ("debug")

# Open the median image
	iferr (im1 = immap (medfile, READ_ONLY, 0)) {
	    call printf ("TCOSLIM: Cannot read %s\n")
		call pargstr (medfile)
	    call clputi ("error", 1)
	    return
	    }
	nc = IM_LEN(im1,1)
	nl = IM_LEN(im1,2)
	if (verbose) {
	    call printf ("TCOSLIM: %d x %d input median image %s opened\n")
		call pargi (nc)
		call pargi (nl)
		call pargstr (medfile)
	    call flush (STDOUT)
	    }

# Read from the median image and process each pixel
	do j = 1, nl {

# Get input buffer for this line of the image
	    linein = imgl2r (im1, j)
	    if (linein == NULL) {
		call printf ("TCOSLIM: Cannot read line %d of %s\n")
		    call pargi (i)
		    call pargstr (medfile)
		call flush (STDOUT)
		call clputi ("error", 3)
		call imunmap (im1)
		return
		}
	    do i = 1, nc {
		datai = Memr[linein+i-1]
		datai0 = datai
		if (datai > 0) {
		    datai = 1.4 * sqrt (datai)
		    if (datai < vmin)
			datai = vmin
		    }
		else
		    datai = vmin
		datalim[i,j] = datai * threshold
		}
	    if (debug) {
		call printf ("TCOSLIM: Line %4d written.\n")
		    call pargi (j)
		call flush (STDOUT)
		}
	    }

# Open the output limit statistics file
	iferr (im2 = immap (limfile, NEW_COPY, im1)) {
	    call printf ("TCOSLIM: Cannot create %s\n")
		call pargstr (limfile)
	    call flush (STDOUT)
	    call clputi ("error", 2)
	    return
	    }
	else if (debug) {
	    call printf ("TCOSLIM: Limit file %s created\n")
		call pargstr (limfile)
	    call flush (STDOUT)
	    }

	call imastr (im2, "i_title", "Statistical limit map")
	call imaddr (im2, "TCOSNSIG", threshold)
	nc = IM_LEN(im2,1)
	nl = IM_LEN(im2,2)
	if (verbose) {
	    call printf ("TCOSLIM: %d x %d output statistics image %s opened\n")
		call pargi (nc)
		call pargi (nl)
		call pargstr (limfile)
	    call flush (STDOUT)
	    }

# Initialize vector for output
	call amovkl (long(1), v, IM_MAXDIM)

# Write the output file one line at a time
	do j = 1, nl {

# Get output buffer for this line of the image
	    v[2] = j
	    if (impnlr (im2, lineout, v) < nc) {
		call printf ("TCOSLIM: Cannot write line %d of %s\n")
		    call pargi (j)
		    call pargstr (limfile)
		    call flush (STDOUT)
		call clputi ("error", 3)
		return
		}
	    call achtrr (datalim[1,j], Memr[lineout], nc)
	    }

# Close output image
	IM_CTIME(im2) = clktime (long(0))
	IM_UPDATE(im2) = YES
	if (verbose) {
	    call printf ("TCOSLIM: Ready to write output image %s\n")
		call pargstr (limfile)
	    call flush (STDOUT)
	    }
	call imunmap (im2)
	if (verbose) {
	    call printf ("TCOSLIM: Output statistics image %s written.\n")
		call pargstr (limfile)
	    call flush (STDOUT)
	    }

# Close input image
	call imunmap (im1)
	if (verbose) {
	    call printf ("TCOSLIM: Input median image %s closed.\n")
		call pargstr (medfile)
	    call flush (STDOUT)
	    }

end
# Sep 24 2007	New program based on hectospec.hcoslim
# Nov  8 2007	Add threshold (=n for n*sigma) to set limit to use
